/*
* This file is part of Qccrypt, an open-source cross-platform frontend for ccrypt
* encryption tool.
* Copyright (C) 2006  Philippe Beaureilles
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact e-mail: Philippe Beaureilles <qccrypt@free.fr>
* Program URL   : http://qccrypt.free.fr/
*
*/

#include <QtGui>
#include <QProcess>
#include <QTabWidget>

#include "ccdlg.h"
#include "newpassworddialog.h"

#define round_up(n, k) (((n) - 1 + (k)) / (k))

CCryptDialog::CCryptDialog(const QString &fileName, QWidget *parent)
    : QDialog(parent)
{
	ccdlg_ui.setupUi(this);
	
	connect(ccdlg_ui.encryptOpenFileNameButton, SIGNAL(clicked()),
			this, SLOT(setEncryptOpenFileName()));
	connect(ccdlg_ui.encryptFileButton, SIGNAL(clicked()),
			this, SLOT(encryptFile()));
	connect(ccdlg_ui.decryptOpenFileNameButton, SIGNAL(clicked()),
			this, SLOT(setDecryptOpenFileName()));
	connect(ccdlg_ui.decryptFileButton, SIGNAL(clicked()),
			this, SLOT(decryptFile()));
	connect(ccdlg_ui.aboutButton, SIGNAL(clicked()),
			this, SLOT(about()));
	connect(ccdlg_ui.encryptFileRadioButton, SIGNAL(clicked()),
			this, SLOT(showEncryptFileGroupBox()));
	connect(ccdlg_ui.encryptFolderRadioButton, SIGNAL(clicked()),
			this, SLOT(showEncryptFolderGroupBox()));
	connect(ccdlg_ui.decryptFileRadioButton, SIGNAL(clicked()),
			this, SLOT(showDecryptFileGroupBox()));
	connect(ccdlg_ui.decryptFolderRadioButton, SIGNAL(clicked()),
			this, SLOT(showDecryptFolderGroupBox()));
	connect(ccdlg_ui.encryptFolderButton, SIGNAL(clicked()),
			this, SLOT(encryptFolder()));
	connect(ccdlg_ui.decryptFolderButton, SIGNAL(clicked()),
			this, SLOT(decryptFolder()));
	connect(ccdlg_ui.encryptOpenFolderNameButton, SIGNAL(clicked()),
			this, SLOT(setEncryptOpenFolderName()));
	connect(ccdlg_ui.decryptOpenFolderNameButton, SIGNAL(clicked()),
			this, SLOT(setDecryptOpenFolderName()));

	showEncryptFileGroupBox();
	showDecryptFileGroupBox();
	emptyAllFields();
	
	if (!fileName.isEmpty())
	{
		currentFileInfo.setFile(fileName);

		if (currentFileInfo.exists())
		{
			if (currentFileInfo.suffix() == "cpt")
				updateFileToDecryptProperties();
			else
				updateFileToEncryptProperties();
    	}
    	else
    	{
    		QMessageBox::warning(this, "Qccrypt", "The file specified as a parameter on the command line did not found!", "&Ok");
    		emptyAllFields();
   		}
	}
	else
   		emptyAllFields();
}

///////////////////////////////
// Display functions
///////////////////////////////
void CCryptDialog::updateFileToEncryptProperties()
{
	if (currentFileInfo.exists())
	{
		ccdlg_ui.tabQccryptMode->setCurrentWidget(ccdlg_ui.encryptionTab);
		ccdlg_ui.encryptFileNameEdit->setText(currentFileInfo.fileName());
		ccdlg_ui.encryptFilePathValueLabel->setText(currentFileInfo.path());
		qlonglong size = round_up(currentFileInfo.size(), 1024);
		ccdlg_ui.encryptFileSizeValueLabel->setText(tr("%1 K").arg(size));
		ccdlg_ui.encryptFileLastReadValueLabel->setText(currentFileInfo.lastRead().toString());
	    ccdlg_ui.encryptFileLastModValueLabel->setText(currentFileInfo.lastModified().toString());
	    ccdlg_ui.encryptFileButton->setEnabled ( true );
	    ccdlg_ui.keepExistingFileCheckBox->setEnabled ( true );
	    emptyDecryptFileFields();
   	}
   	else
   	{
   		emptyEncryptFileFields();
   		emptyDecryptFileFields();
  	}
}

void CCryptDialog::updateFileToDecryptProperties()
{
	if (currentFileInfo.exists())
	{
		ccdlg_ui.tabQccryptMode->setCurrentWidget(ccdlg_ui.decryptionTab);
		ccdlg_ui.decryptFileNameEdit->setText(currentFileInfo.fileName());
		ccdlg_ui.decryptFilePathValueLabel->setText(currentFileInfo.path());
		qlonglong size = round_up(currentFileInfo.size(), 1024);
		ccdlg_ui.decryptFileSizeValueLabel->setText(tr("%1 K").arg(size));
		ccdlg_ui.decryptFileLastReadValueLabel->setText(currentFileInfo.lastRead().toString());
	    ccdlg_ui.decryptFileLastModValueLabel->setText(currentFileInfo.lastModified().toString());
	    ccdlg_ui.decryptFileButton->setEnabled ( true );
	    ccdlg_ui.keepExistingEncryptedFileCheckBox->setEnabled ( true );
	    emptyEncryptFileFields();
   	}
   	else
   	{
   		emptyEncryptFileFields();
   		emptyDecryptFileFields();
  	}
}

void CCryptDialog::updateFolderToEncryptProperties()
{
	if (currentFolderInfo.exists())
	{
		ccdlg_ui.tabQccryptMode->setCurrentWidget(ccdlg_ui.encryptionTab);
		ccdlg_ui.encryptFolderNameEdit->setText(currentFolderInfo.path());
		ccdlg_ui.encryptFolderSizeValueLabel->setText(tr("%1 Kbytes").arg(round_up(getFolderSize(currentFolderInfo.path()),1024)));
		ccdlg_ui.encryptFolderSubFoldersCountValueLabel->setText(tr("%1").arg(subFoldersCount(currentFolderInfo.path())));
		ccdlg_ui.encryptFolderSubFilesCountValueLabel->setText(tr("%1").arg(subFilesCount(currentFolderInfo.path())));
    	ccdlg_ui.encryptFolderButton->setEnabled ( true );
    	emptyDecryptFolderFields();
	}
	else
	{
   		emptyEncryptFolderFields();
   		emptyDecryptFolderFields();
	}
}

void CCryptDialog::updateFolderToDecryptProperties()
{
	if (currentFolderInfo.exists())
	{
		ccdlg_ui.tabQccryptMode->setCurrentWidget(ccdlg_ui.decryptionTab);
		ccdlg_ui.decryptFolderNameEdit->setText(currentFolderInfo.path());
		ccdlg_ui.decryptFolderSizeValueLabel->setText(tr("%1 Kbytes").arg(round_up(getFolderSize(currentFolderInfo.path()),1024)));
		ccdlg_ui.decryptFolderSubFoldersCountValueLabel->setText(tr("%1").arg(subFoldersCount(currentFolderInfo.path())));
		ccdlg_ui.decryptFolderSubFilesCountValueLabel->setText(tr("%1").arg(subFilesCount(currentFolderInfo.path())));
    	ccdlg_ui.decryptFolderButton->setEnabled ( true );
    	emptyEncryptFolderFields();
	}
	else
	{
   		emptyEncryptFolderFields();
   		emptyDecryptFolderFields();
	}
}

void CCryptDialog::emptyEncryptFileFields()
{
	ccdlg_ui.encryptFileNameEdit->setText("");
	ccdlg_ui.encryptFilePathValueLabel->setText("");
	ccdlg_ui.encryptFileSizeValueLabel->setText("");
	ccdlg_ui.encryptFileLastReadValueLabel->setText("");
    ccdlg_ui.encryptFileLastModValueLabel->setText("");
	ccdlg_ui.encryptFileButton->setEnabled ( false );
    ccdlg_ui.keepExistingFileCheckBox->setEnabled ( false );
}

void CCryptDialog::emptyDecryptFileFields()
{
	ccdlg_ui.decryptFileNameEdit->setText("");
	ccdlg_ui.decryptFilePathValueLabel->setText("");
	ccdlg_ui.decryptFileSizeValueLabel->setText("");
	ccdlg_ui.decryptFileLastReadValueLabel->setText("");
    ccdlg_ui.decryptFileLastModValueLabel->setText("");
	ccdlg_ui.decryptFileButton->setEnabled ( false );
    ccdlg_ui.keepExistingEncryptedFileCheckBox->setEnabled ( false );
}

void CCryptDialog::emptyEncryptFolderFields()
{
	ccdlg_ui.encryptFolderNameEdit->setText("");
	ccdlg_ui.encryptFolderSizeValueLabel->setText("");
	ccdlg_ui.encryptFolderSubFoldersCountValueLabel->setText("");
    ccdlg_ui.encryptFolderSubFilesCountValueLabel->setText("");
	ccdlg_ui.encryptFolderButton->setEnabled ( false );
}

void CCryptDialog::emptyDecryptFolderFields()
{
	ccdlg_ui.decryptFolderNameEdit->setText("");
	ccdlg_ui.decryptFolderSizeValueLabel->setText("");
	ccdlg_ui.decryptFolderSubFoldersCountValueLabel->setText("");
    ccdlg_ui.decryptFolderSubFilesCountValueLabel->setText("");
	ccdlg_ui.decryptFolderButton->setEnabled ( false );
}

void CCryptDialog::emptyAllFields()
{
	emptyEncryptFileFields();
	emptyEncryptFolderFields();
	emptyDecryptFileFields();
	emptyDecryptFolderFields();
}

void CCryptDialog::showEncryptFileGroupBox()
{
	ccdlg_ui.encryptFolderGroupBox->hide();
	ccdlg_ui.encryptFileGroupBox->show();
}

void CCryptDialog::showDecryptFileGroupBox()
{
	ccdlg_ui.decryptFolderGroupBox->hide();
	ccdlg_ui.decryptFileGroupBox->show();
}

void CCryptDialog::showEncryptFolderGroupBox()
{
	ccdlg_ui.encryptFileGroupBox->hide();
	ccdlg_ui.encryptFolderGroupBox->show();
}

void CCryptDialog::showDecryptFolderGroupBox()
{
	ccdlg_ui.decryptFileGroupBox->hide();
	ccdlg_ui.decryptFolderGroupBox->show();
}

qlonglong CCryptDialog::getFolderSize(QString path)
{
	qlonglong size=0;
	QDir folder;
	folder.setPath(path);
	QFileInfoList fileInfoList = folder.entryInfoList(QDir::AllEntries|QDir::NoDotAndDotDot);
	for(int i = 0; i < fileInfoList.size(); ++i)
	{
		if(fileInfoList.at(i).isDir())
			size+=getFolderSize(fileInfoList.at(i).path() + "/" + fileInfoList.at(i).fileName());
		else
	  		size+=fileInfoList.at(i).size();
	}
	return size;
}

qlonglong CCryptDialog::subFoldersCount(QString path)
{
	qlonglong count=0;
	QDir folder;
	folder.setPath(path);
	QFileInfoList fileInfoList = folder.entryInfoList(QDir::AllEntries|QDir::NoDotAndDotDot);
	for(int i = 0; i < fileInfoList.size(); ++i)
	{
		if(fileInfoList.at(i).isDir())
		{
			count++;
			count+=subFoldersCount(fileInfoList.at(i).path() + "/" + fileInfoList.at(i).fileName());
		}
	}
	return count;
}

qlonglong CCryptDialog::subFilesCount(QString path)
{
	qlonglong count=0;
	QDir folder;
	folder.setPath(path);
	QFileInfoList fileInfoList = folder.entryInfoList(QDir::AllEntries|QDir::NoDotAndDotDot);
	for(int i = 0; i < fileInfoList.size(); ++i)
	{
		if (fileInfoList.at(i).isFile())
			count++;
		if (fileInfoList.at(i).isDir())
			count+=subFilesCount(fileInfoList.at(i).path() + "/" + fileInfoList.at(i).fileName());
	}
	return count;
}

///////////////////////////////
// Connectors
///////////////////////////////
void CCryptDialog::setEncryptOpenFileName()
{
	QString fileName = QFileDialog::getOpenFileName(this,
								tr("Select the file to encrypt"),
								ccdlg_ui.encryptFileNameEdit->text(),
								tr("All Files (*)"));
	currentFileInfo.setFile(fileName);
	updateFileToEncryptProperties();
}

void CCryptDialog::encryptFile()
{
	NewPasswordDialog pwDlg(this);
	if (pwDlg.exec() == QDialog::Accepted)
	{
		QString password = pwDlg.getPassword();
		
		if (!password.isEmpty())
		{
			QString targetFileName;
			targetFileName = QString(currentFileInfo.absoluteFilePath());
			targetFileName = targetFileName + ".cpt";
			QFileInfo targetFile;
			targetFile = QFileInfo(targetFileName);

			if (targetFile.isFile())
			{
				int reply = QMessageBox::warning(this, tr("Replace file confirmation"),
								tr("The target file already exists. Do you want to overwrite it ?"),
								tr("&Yes"),
								tr("&No"));
				if (reply == 1)
					return;
			}
			QString program = "ccrypt";
			QStringList arguments;
			QProcess *encryptProcess = new QProcess(this);
			
			if (ccdlg_ui.keepExistingFileCheckBox->isChecked())
			{
				arguments << "-e" << "-f" << "-K" << password;
				encryptProcess->setStandardInputFile(QString(currentFileInfo.absoluteFilePath()));
				encryptProcess->setStandardOutputFile(QString(currentFileInfo.absoluteFilePath() + ".cpt"));
			}
			else
				arguments << "-e" << "-f" << "-K" << password << currentFileInfo.absoluteFilePath() ;
			encryptProcess->start(program, arguments);
			encryptProcess->waitForFinished();
			int returnCode = encryptProcess->exitCode ();
	
			switch (returnCode) {
				case 0 :
					QMessageBox::information(this, "Qccrypt", "Encryption successful.", "&Ok");
					emptyAllFields();
					break;
				default :
					QMessageBox::critical(this, "Qccrypt", "Encryption error.", "&Ok");
			}
		}
		else
			QMessageBox::warning(this, "Qccrypt", "You must enter a password to encrypt this file!", "&Ok");
	}
}


void CCryptDialog::setDecryptOpenFileName()
{
	QString fileName = QFileDialog::getOpenFileName(this,
								tr("Select the file to decrypt"),
								ccdlg_ui.encryptFileNameEdit->text(),
								tr("ccrypt crypted file (*.cpt)"));
	currentFileInfo.setFile(fileName);
	updateFileToDecryptProperties();
}

void CCryptDialog::decryptFile()
{
	bool ok;
	QString password = QInputDialog::getText(this, tr("Password"),
										tr("Enter the password:"), QLineEdit::Password,
										"", &ok);
	if (ok)
	{
		if (!password.isEmpty())
		{
			QString targetFileName;
			targetFileName = QString(currentFileInfo.absoluteFilePath());
			targetFileName = targetFileName.left(targetFileName.length() - 4);
			QFileInfo targetFile;
			targetFile = QFileInfo(targetFileName);

			if (targetFile.isFile())
			{
				int reply = QMessageBox::warning(this, tr("Replace file confirmation"),
								tr("The target file already exists. Do you want to overwrite it ?"),
								tr("&Yes"),
								tr("&No"));
				if (reply == 1)
					return;
			}

			QString program = "ccrypt";
			QStringList arguments;
			QProcess *encryptProcess = new QProcess(this);
			
			if (ccdlg_ui.keepExistingEncryptedFileCheckBox->isChecked())
			{
				arguments << "-d" << "-f" << "-K" << password;
				QString fileName = QString(currentFileInfo.absoluteFilePath());
				encryptProcess->setStandardInputFile(fileName);
				fileName.truncate(fileName.length() - 4);
				encryptProcess->setStandardOutputFile(fileName);
			}
			else
				arguments << "-d" << "-f" << "-K" << password << currentFileInfo.absoluteFilePath() ;
			encryptProcess->start(program, arguments);
			encryptProcess->waitForFinished();
			int returnCode = encryptProcess->exitCode ();
			
			switch (returnCode) {
				case 0 :
					QMessageBox::information(this, "Qccrypt", "Decryption successful.", "&Ok");
					emptyAllFields();
					break;
				case 4 :
					QMessageBox::critical(this, "Qccrypt", "The password specified was incorrect!", "&Ok");
					break;
				default :
					QMessageBox::critical(this, "Qccrypt", "Decryption error.", "&Ok");
			}
		}
		else
			QMessageBox::warning(this, "Qccrypt", "You must enter the password to decrypt this file!", "&Ok");
	}
}

void CCryptDialog::about()
{
	QMessageBox::information(this, "About Qccrypt 0.2", "Qccrypt is an Open Source frontend for ccrypt encryption/decryption tool.\nIt is written in C++ with QT4 libraries by Philippe Beaureilles.\n\nThis program is free software. You can redistribute it and/or modify it under the terms of\nthe GNU General Public License as published by the Free Software Foundation\neither version 2 of the License, or (at your option) any later version.\n\nhttp://qccrypt.free/", "&Ok");
}


void CCryptDialog::encryptFolder()
{
	NewPasswordDialog pwDlg(this);
	if (pwDlg.exec() == QDialog::Accepted)
	{
		QString password = pwDlg.getPassword();
		
		if (!password.isEmpty())
		{
				
			QString program = "ccrypt";
			QStringList arguments;
			arguments << "-e" << "-r" << "-K" << password << currentFolderInfo.path();
			QProcess *encryptProcess = new QProcess(this);
			encryptProcess->start(program, arguments);
			encryptProcess->waitForFinished();
			int returnCode = encryptProcess->exitCode ();

			switch (returnCode) {
				case 0 :
					QMessageBox::information(this, "Qccrypt", "Encryption successful.", "&Ok");
					emptyAllFields();
					break;
				default :
					QMessageBox::critical(this, "Qccrypt", "Encryption error.", "&Ok");
			}
		}
		else
			QMessageBox::warning(this, "Qccrypt", "You must enter a password to encrypt this folder!", "&Ok");
	}	
}

void CCryptDialog::decryptFolder()
{
	bool ok;
	QString password = QInputDialog::getText(this, tr("Password"),
										tr("Enter the password:"), QLineEdit::Password,
										"", &ok);
	if (ok)
	{
		if (!password.isEmpty())
		{
				
			QString program = "ccrypt";
			QStringList arguments;
			arguments << "-d" << "-r" << "-K" << password << currentFolderInfo.path() ;
			QProcess *decryptProcess = new QProcess(this);
			decryptProcess->start(program, arguments);
			decryptProcess->waitForFinished();
			int returnCode = decryptProcess->exitCode();

			switch (returnCode) {
				case 0 :
					QMessageBox::information(this, "Qccrypt", "Decryption successful.", "&Ok");
					emptyAllFields();
					break;
				default :
					QMessageBox::information(this, "Qccrypt", "Decryption is finished, but and error code has been reported.<br>Perhaps that some files were not encrypted or were encrypted with a different password.", "&Ok");
			}
		}
		else
			QMessageBox::warning(this, "Qccrypt", "You must enter a password to decrypt this folder!", "&Ok");
	}
}

void CCryptDialog::setEncryptOpenFolderName()
{
	QString folderName = QFileDialog::getExistingDirectory(this,
								tr("Select the folder to encrypt"),
								ccdlg_ui.encryptFolderNameEdit->text(),
								QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	currentFolderInfo.setPath(folderName);
	updateFolderToEncryptProperties();
}

void CCryptDialog::setDecryptOpenFolderName()
{
	QString folderName = QFileDialog::getExistingDirectory(this,
								tr("Select the folder to decrypt"),
								ccdlg_ui.decryptFolderNameEdit->text(),
								QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	currentFolderInfo.setPath(folderName);
	updateFolderToDecryptProperties();
}

